/*********************************************************************
*
* Software License Agreement (BSD License)
*
*  Copyright (c) 2008, Willow Garage, Inc.
*  All rights reserved.
*
*  Redistribution and use in source and binary forms, with or without
*  modification, are permitted provided that the following conditions
*  are met:
*
*   * Redistributions of source code must retain the above copyright
*     notice, this list of conditions and the following disclaimer.
*   * Redistributions in binary form must reproduce the above
*     copyright notice, this list of conditions and the following
*     disclaimer in the documentation and/or other materials provided
*     with the distribution.
*   * Neither the name of the Willow Garage nor the names of its
*     contributors may be used to endorse or promote products derived
*     from this software without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
*  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
*  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
*  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
*  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
*  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
*  POSSIBILITY OF SUCH DAMAGE.
*
* Author: Eitan Marder-Eppstein
*********************************************************************/
#ifndef COSTMAP_CELL_DATA_H_
#define COSTMAP_CELL_DATA_H_
namespace costmap_2d {
  /**
   * @class CellData
   * @brief Storage for cell information used during obstacle inflation
   */
  class CellData {
    public:
      /**
       * @brief  Constructor for a CellData object
       * @param  d The distance to the nearest obstacle, used for ordering in the priority queue
       * @param  i The index of the cell in the cost map
       * @param  x The x coordinate of the cell in the cost map
       * @param  y The y coordinate of the cell in the cost map
       * @param  sx The x coordinate of the closest obstacle cell in the costmap
       * @param  sy The y coordinate of the closest obstacle cell in the costmap
       * @return 
       */
      CellData(double d, double i, unsigned int x, unsigned int y, unsigned int sx, unsigned int sy) : distance_(d), 
      index_(i), x_(x), y_(y), src_x_(sx), src_y_(sy) {}
      double distance_;
      unsigned int index_;
      unsigned int x_, y_;
      unsigned int src_x_, src_y_;
  };

  /**
   * @brief Provide an ordering between CellData objects in the priority queue 
   * @return We want the lowest distance to have the highest priority... so this returns true if a has higher priority than b
   */
  inline bool operator<(const CellData &a, const CellData &b){
    return a.distance_ > b.distance_;
  }
};
#endif
